/**
 * License Agreement.
 *
 *  JBoss RichFaces 3.0 - Ajax4jsf Component Library
 *
 * Copyright (C) 2007  Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

package org.richfaces.component;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * @author Nick Belaevski - nbelaevski@exadel.com
 * created 17.11.2006
 * Default {@link TreeRowKey} implementation based on {@link ArrayList}
 */
public class ListRowKey extends TreeRowKey {

	private ArrayList path;

	/**
	 * 
	 */
	private static final long serialVersionUID = 7718335783201397177L;

	public String toString() {
		return getPath();
	}

	/**
	 * Default constructor
	 */
	public ListRowKey() {
		super();
		this.path = new ArrayList();
	}

	/**
	 * Copy constructor
	 * @param parentRowKey row key to clone
	 */
	protected ListRowKey(ListRowKey parentRowKey) {
		super();
		this.path = (ArrayList) parentRowKey.path.clone();
	}

	/**
	 * Appending constructor
	 * @param parentRowKey base row key
	 * @param pathElement path segment to append to base row key
	 */
	public ListRowKey(ListRowKey parentRowKey, Object pathElement) {
		this(parentRowKey);
		this.path.add(pathElement);
	}

	private static ArrayList parsePath(String path) {
		ArrayList result = new ArrayList();

		String trimmedPath = path.trim();

		int idx = -1;
		int oldIdx = 0;

		do {
			idx = trimmedPath.indexOf(AbstractTreeDataModel.SEPARATOR, oldIdx);

			String subData;

			if (idx != -1) {
				subData = trimmedPath.substring(oldIdx, idx);
			} else {
				subData = trimmedPath.substring(oldIdx);
			}

			subData = subData.trim();
			if (subData.length() != 0) {
				result.add(subData);
			}

			oldIdx = idx + 1;
		} while (idx != -1);

		return result;
	}

	/**
	 * List constructor
	 * @param list List of strings to create corresponding row key from
	 */
	public ListRowKey(List list) {
		super();

		this.path = new ArrayList(list);
	}
	
	/**
	 * Path object constructor
	 * @param path first path segment
	 */
	public ListRowKey(Object path) {
		super();
		this.path = new ArrayList(1);
		this.path.add(path);
	}

	/**
	 * Path string constructor
	 * @param path path string to create corresponding row key from
	 */
	public ListRowKey(String path) {
		super();
		this.path = parsePath(path);
	}

	public int depth() {
		return path.size();
	}

	public Iterator iterator() {
		return path.iterator();
	}

	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((path == null) ? 0 : path.hashCode());
		return result;
	}

	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final ListRowKey other = (ListRowKey) obj;
		if (path == null) {
			if (other.path != null)
				return false;
		} else if (!path.equals(other.path))
			return false;
		return true;
	}

	public Iterator getSubPathIterator(int fromIndex) {
		return path.listIterator(fromIndex);
	}

	public boolean isSubKey(TreeRowKey rowKey) {
		if (rowKey instanceof ListRowKey) {
			ListRowKey listRowKey = (ListRowKey) rowKey;

			if (listRowKey == null) {
				return false;
			}

			int commonLength = getCommonPathLength(listRowKey);
			if (commonLength != 0) {
				return commonLength == depth();
			} else {
				return rowKey.depth() == 0;
			}
		} else {
			return super.isSubKey(rowKey);
		}
	}

	public String getPath() {
		StringBuffer result = new StringBuffer();
		Iterator iterator = path.iterator();
		boolean hasNext = iterator.hasNext();

		while (hasNext) {
			result.append(iterator.next());
			hasNext = iterator.hasNext();

			if (hasNext) {
				result.append(AbstractTreeDataModel.SEPARATOR);
			}
		}

		return result.toString();
	}

	public int getCommonPathLength(TreeRowKey otherRowKey) {
		if (otherRowKey == null)
			return 0;
		Iterator iterator = this.iterator();
		Iterator otherIterator = otherRowKey.iterator();
		int length = 0;
		while (iterator.hasNext() && otherIterator.hasNext()
				&& iterator.next().equals(otherIterator.next()))
			length++;
		return length;
	}
}
